// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "sim.h"
#include "dump.h"
#include "inp.h"

void dump_load_config(struct device *in)
{
	int dump;
	struct inp_file inp;
	inp_init(&inp);
	inp_load_from_path(&inp,in->inputpath,"dump.inp");
	inp_check(&inp,1.32);

	inp_search_int(&inp,&(dump),"#plot");
	set_dump_status(dump_plot,dump);

	inp_search_int(&inp,&(dump),"#newton_dump");
	set_dump_status(dump_newton, dump);

	inp_search_int(&inp,&(dump),"#dump_dynamic");
	set_dump_status(dump_dynamic, dump);

	inp_search_int(&inp,&(in->stop_start),"#startstop");

	inp_search_int(&inp,&(in->dumpitdos),"#dumpitdos");

	inp_search_string(&inp,in->plot_file,"#plotfile");

	inp_search_double(&inp,&(in->start_stop_time),"#start_stop_time");

	inp_search_int(&inp,&(dump),"#dump_iodump");
	set_dump_status(dump_iodump,dump);

	inp_search_int(&inp,&(in->dump_movie),"#dump_movie");

	inp_search_int(&inp,&(dump),"#dump_optics");
	set_dump_status(dump_optics, dump);

	inp_search_int(&inp,&(dump),"#dump_optics_verbose");
	set_dump_status(dump_optics_verbose, dump);

	inp_search_int(&inp,&(dump),"#dump_norm_time_to_one");
	set_dump_status(dump_norm_time_to_one, dump);

	inp_search_int(&inp,&(dump),"#dump_norm_y_axis");
	set_dump_status(dump_norm_y_axis, dump);

	inp_search_int(&inp,&(dump),"#dump_1d_slices");
	set_dump_status(dump_1d_slices, dump);

	inp_search_int(&inp,&(dump),"#dump_energy_slice_switch");
	set_dump_status(dump_energy_slice_switch, dump);

	inp_search_int(&inp,&(in->dump_slicepos),"#dump_energy_slice_pos");
	if (in->dump_slicepos>=in->ymeshpoints) in->dump_slicepos=0;

	inp_search_int(&inp,&(dump),"#dump_print_newtonerror");
	set_dump_status(dump_print_newtonerror, dump);

	inp_search_int(&inp,&(dump),"#dump_write_converge");
	set_dump_status(dump_write_converge, dump);

	inp_search_int(&inp,&(dump),"#dump_print_converge");
	set_dump_status(dump_print_converge, dump);

	inp_search_int(&inp,&(dump),"#dump_print_pos_error");
	set_dump_status(dump_print_pos_error, dump);

	inp_search_int(&inp,&(dump),"#dump_pl");
	set_dump_status(dump_pl, dump);

	inp_search_int(&inp,&(dump),"#dump_zip_files");
	set_dump_status(dump_zip_files, dump);

	inp_search_int(&inp,&(dump),"#dump_print_hardware_info");
	set_dump_status(dump_print_hardware_info, dump);


	inp_free(&inp);

	if (get_dump_status(dump_iodump)==FALSE)
	{
		in->dumpitdos=FALSE;
		set_dump_status(dump_optics,FALSE);
	}

}
