// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"
double lib_error(struct fitvars *config,int ext,char * path)
{
double mul;
char expfile[1000];
char simfile[1000];
char out_name_exp[1000];
char out_name_sim[1000];
char out_name_delta[1000];

char config_file[1000];
sprintf(expfile,"%s./exp/jv%d/jv.dat",path,ext);
sprintf(simfile,"%s./sim/jv%d/jvexternal.dat",path,ext);
sprintf(config_file,"%s./exp/jv%d/fit.inp",path,ext);
sprintf(out_name_exp,"./error_jv_exp%d.dat",ext);
sprintf(out_name_sim,"./error_jv_sim%d.dat",ext);
sprintf(out_name_delta,"./error_jv_delta%d.dat",ext);

double error=0.0;
struct istruct exp;
struct istruct sim;
struct istruct mul_data;

inter_load(&exp,expfile);
inter_load(&sim,simfile);


double max=0.0;
char tempstring[100];
double start=0.0;
double stop=0.8;
int light=0;
FILE *info=fopen(config_file,"r");
if (info==NULL)
{
	printf("Problem with %s\n",config_file);
	getchar();
}
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&start);
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&stop);
fscanf(info,"%s",tempstring);
fscanf(info,"%d",&light);
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&mul);
fscanf(info,"%s",tempstring);

if (strcmp("#end",tempstring)!=0)
{
	printf("Problem with fit.inp in %s!\n","./exp/jv/");
	getchar();
}
fclose(info);

inter_copy(&mul_data,&exp,TRUE);
inter_set_value(&mul_data,1.0);

if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];

if (stop>exp.x[exp.len-1]) stop=exp.x[exp.len-1];
if (stop>sim.x[sim.len-1]) stop=sim.x[sim.len-1];

inter_chop(&exp,start,stop);
inter_chop(&sim,start,stop);

if (light==0)
{
inter_log_y(&sim);
inter_log_y(&exp);
}

double min=inter_get(&exp,start);

inter_sub_double(&exp,min);
inter_sub_double(&sim,min);

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


inter_save(&exp,out_name_exp);
inter_save(&sim,out_name_sim);

error=inter_get_diff(out_name_delta,&exp,&sim,start,stop,&mul_data);

inter_free(&exp);
inter_free(&sim);
inter_free(&mul_data);
return error;
}
