// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"

double lib_error(struct fitvars *config,char* name,char * path)
{
char exp_name[40];
char sim_name[40];
char fit_config_name[40];
double error=0.0;
char temp[1000];
double ver;
double mul;

FILE *file;
sprintf(fit_config_name,"%s./exp/%s/fit.inp",path,name);


file=fopen(fit_config_name,"r");
fscanf(file,"%s",temp);
fscanf(file,"%le",&mul);

fscanf(file,"%s",temp);
fscanf(file,"%le",&ver);

if (ver!=1.0)
{
	printf("File compatability problem %s\n",fit_config_name);
	getchar();
}

fscanf(file,"%s",temp);
if (strcmp("#end",temp)!=0)
{
	printf("Problem with %s file!\n",fit_config_name);
	getchar();
}
fclose(file);

sprintf(exp_name,"%s./exp/%s/reflect.dat",path,name);
sprintf(sim_name,"%s./sim/%s/reflect.dat",path,name);

struct istruct exp;
struct istruct sim;

inter_load(&exp,exp_name);
inter_load(&sim,sim_name);

inter_sort(&exp);

inter_purge_zero(&sim);
inter_purge_zero(&exp);


double e0=fabs(inter_get_hard(&sim,480e-9)-inter_get_hard(&exp,480e-9))*mul;
double e1=fabs(inter_get_hard(&sim,560e-9)-inter_get_hard(&exp,560e-9))*mul;
double e2=fabs(inter_get_hard(&sim,500e-9)-inter_get_hard(&exp,500e-9))*mul;
double e3=fabs(inter_get_hard(&sim,600e-9)-inter_get_hard(&exp,600e-9))*mul;
double e4=fabs(inter_get_hard(&sim,390e-9)-inter_get_hard(&exp,390e-9))*mul;

printf("390 nm e4=%lf\n",e4);
printf("480 nm e0=%lf\n",e0);
printf("500 nm e2=%lf\n",e2);
printf("560 nm e1=%lf\n",e1);
printf("600 nm e3=%lf\n",e3);

error=(e0+e1+e2+e3+e4)/5;


inter_free(&exp);
inter_free(&sim);
return error;
}
