// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "ce.h"


//    ----  ----
//    |        |
//    |        |
//   SC        R
//    |        |
//    |	       |
//    ----  ----
//So we hand the solver everyting as if we were going to do current and it hads it back as it wasnts.
void newton_aux_ce(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
//<clean>
double C=in->C;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
/*
double S=((in->Rshort+in->Rcontact)-in->Rshunt)/(in->Rshunt*(in->Rshort+in->Rcontact));

*i=i0+C*(V-in->Vapplied_last)/in->dt-(V)*S;
*didv=didv0+C*(1.0)/in->dt-S;
*didphi=didphi0;
*didxil=didxil0;
*didxipl=didxipl0;
*didphir=didphir0;
*didxir=didxir0;
*didxipr=didxipr0;
*/
double S=1.0/(in->Rshort+in->Rcontact);
double L=ce_config.ce_L;
*i=i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt+(V-L*(i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt-in->ilast)/in->dt)*S;
*didv=didv0+C*(1.0)/in->dt+1.0/in->Rshunt+(1.0-L*(didv0+C*(1.0)/in->dt+1.0/in->Rshunt)/in->dt)*S;//didv0+C*(1.0)/in->dt-S;
*didphi=didphi0+(-L*(didphi0)/in->dt)*S;//didphi0;
*didxil=didxil0+(-L*(didxil0)/in->dt)*S;//didxil0;
*didxipl=didxipl0+(-L*(didxipl0)/in->dt)*S;//didxipl0;
*didphir=didphir0+(-L*(didphir0)/in->dt)*S;//didphir0;
*didxir=didxir0+(-L*(didxir0)/in->dt)*S;//didxir0;
*didxipr=didxipr0+(-L*(didxipr0)/in->dt)*S;//didxipr0;


//</clean>
return;
}

double newton_charge_extraction(struct device *in)
{
//<clean>
in->kl_in_newton=TRUE;
solver_realloc(in);
in->newton_aux=&newton_aux_ce;
solve_all(in);
//</clean>
return 0.0;
}
