// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../sim.h"
#include "jv_simple.h"
#include "dump.h"
#include <math.h>
#include "../../elec_plugins.h"
#include "ntricks.h"
#include "gui_hooks.h"
#include "../../inp.h"

static int unused __attribute__((unused));

void sim_jv_simple(struct device *in)
{
gui_start();
struct jv_simple config;
jv_simple_load_config(&config,in);
double V=0.0;
double Vstop=config.Vstop;
double Vstep=config.Vstep;
int ittr=0;
int first=TRUE;
double Vlast;
double Jlast;
double Vexternal;

int stop=FALSE;
struct istruct ivexternal;
inter_init(&ivexternal);

struct istruct jvexternal;
inter_init(&jvexternal);

V=in->Vapplied;

in->Vapplied=config.Vstart;


double i=0.0;
double J=0.0;
double T=in->Te[0];
printf ("JV simple\n");
	do
	{
		i=config.I0*(exp(Q*V/(config.nid*kb*T)))+V/in->Rshunt;

		Vexternal=i*in->Rcontact+V;

		J=i/(in->xlen*in->zlen);

		ittr++;
			inter_append(&jvexternal,Vexternal,J);
			inter_append(&ivexternal,Vexternal,i);

			if (first==FALSE)
			{

				//check if we have crossed 0V
				if ((Vlast<=0)&&(Vexternal>=0.0))
				{
					in->Jsc=Jlast+(J-Jlast)*(0-Vlast)/(V-Vlast);
					printf("Jsc = %e\n",in->Jsc);
				}

				if ((Jlast<=0)&&(J>=0.0))
				{
					in->Voc=Vlast+(Vexternal-Vlast)*(0-Jlast)/(J-Jlast);
					printf("Voc = %e\n",in->Voc);
				}



				if (Vexternal>Vstop) break;

			}

			if (get_dump_status(dump_iodump)==TRUE)
			{
			printf("V=%lf Vexternal=%lf J = %e A, i= %le A\n",V,Vexternal,J,i);
			}


			Jlast=J;
			Vlast=Vexternal;
			first=FALSE;

			V+=Vstep;
			//dialog_set_progress ((in->Vstart+V)/(in->Vstop-in->Vstart));

			if ((Vstep>=0)&&(V>Vstop))
			{
				stop=TRUE;
				printf("--------------1\n");
			}

			if ((Vstep<0)&&(V<Vstop))
			{
				stop=TRUE;
				printf("--------------2\n");
			}

		if (stop==TRUE)
		{
			break;
		}


	}while(1);

in->FF=fabs(in->Pmax/(in->Jsc*in->Voc));

if (get_dump_status(dump_print_text)==TRUE)
{
	printf("Voc= %lf (V)\n",in->Voc);
	printf("Jsc= %lf (A/m^2)\n",in->Jsc);
	printf("Pmax= %lf (W/m^2)\n",in->Pmax);
	printf("Voltage to get Pmax= %lf (V)\n",in->Pmax_voltage);
	printf("FF= %lf\n",in->FF*100.0);
	printf("Efficiency= %lf percent\n",fabs(in->Pmax/in->Psun)*100.0);
}


	FILE *out;
	out=fopena(in->outputpath,"./sim_info.dat","w");
	fprintf(out,"#ff\n%le\n",in->FF);
	fprintf(out,"#pce\n%le\n",fabs(in->Pmax/(in->Psun))*100.0);
	fprintf(out,"#voc\n%le\n",in->Voc);
	fprintf(out,"#jsc\n%le\n",in->Jsc);
	fprintf(out,"#end");
	fclose(out);


inter_save_a(&ivexternal,in->outputpath,"ivexternal.dat");
inter_free(&ivexternal);

inter_save_a(&jvexternal,in->outputpath,"jvexternal.dat");
inter_free(&jvexternal);


gui_stop();
}




void jv_simple_load_config(struct jv_simple* in,struct device *dev)
{
struct inp_file inp;
inp_init(&inp);
inp_load_from_path(&inp,dev->inputpath,"jv_simple.inp");
inp_check(&inp,1.0);
inp_search_double(&inp,&(in->nid),"#nid");
inp_search_double(&inp,&(in->I0),"#I0");
inp_search_double(&inp,&(in->Vstep),"#Vstep");
inp_search_double(&inp,&(in->Vstart),"#Vstart");
inp_search_double(&inp,&(in->Vstop),"#Vstop");
inp_free(&inp);

}
