// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "dump.h"
#include "inp.h"
#include "buffer.h"
#include "light_interface.h"
#include "../../dynamic_store.h"
#include "../../light.h"
#include "../../buffer.h"
#include "optics_plugin.h"

static int unused __attribute__((unused));

void sim_optics(struct device *in)
{
	struct buffer buf;
	buffer_init(&buf);

	struct inp_file inp;
	struct light two;
	light_init(&two,in,in->outputpath);
	//light_set_dx(&cell.mylight,cell.ymesh[1]-cell.ymesh[0]);
	light_load_config(&two);
	two.disable_transfer_to_electrical_mesh=TRUE;

	double Psun;
	inp_init(&inp);
	inp_load_from_path(&inp,in->outputpath,"light.inp");
	inp_search_double(&inp,&(Psun),"#Psun");
	Psun=fabs(Psun);
	inp_free(&inp);

	light_solve_and_update(in,&two,Psun,0.0,0.0);
	//light_dump(&two);

	buffer_malloc(&buf);
	buf.y_mul=1.0;
	buf.x_mul=1e9;
	strcpy(buf.title,"Wavelength - Reflected light");
	strcpy(buf.type,"xy");
	strcpy(buf.x_label,"Wavelength");
	strcpy(buf.y_label,"Reflected light");
	strcpy(buf.x_units,"nm");
	strcpy(buf.y_units,"a.u.");
	buf.logscale_x=0;
	buf.logscale_y=0;
	buffer_add_info(&buf);
	buffer_add_xy_data(&buf,two.l, two.reflect, two.lpoints);
	buffer_dump(in->outputpath,"reflect.dat",&buf);
	buffer_free(&buf);

	light_free(&two);
	complex_solver_free();
}
