// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "pulse_voc.h"
#include "../../inp.h"
#include "../i/i.h"
#include "../../buffer.h"
#include "../../dynamic_store.h"

static int unused __attribute__((unused));

struct pulse_voc pulse_voc_config;

void sim_pulse_voc(struct device *in)
{
double V=0.0;
double ratio=0.0;
struct buffer buf;
buffer_init(&buf);

pulse_voc_load_config(&pulse_voc_config,in);

in->dt=pulse_voc_config.pulse_voc_dt;



in->go_time=FALSE;


in->externalv=pulse_voc_config.pulse_voc_Vexternal;
printf("Running pulse_voc at %ld Suns\n",in->Psun);
int ittr=0;

int step=0;
light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

sim_externalv(in,in->externalv);

remesh_reset(in,in->externalv);

light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);
//printf("rod %le %le\n",in->Vapplied,in->Gn[0]);
//getchar();

newton_pulse_voc(in,FALSE);
device_timestep(in);



newton_pulse_voc(in,FALSE);
device_timestep(in);

in->time=pulse_voc_config.pulse_voc_time_start;

time_init(in);

device_timestep(in);

in->go_time=TRUE;


struct dynamic_store store;
dump_dynamic_init(&store,in);

struct istruct out_i;
inter_init(&out_i);

struct istruct out_v;
inter_init(&out_v);

struct istruct lost_charge;
inter_init(&lost_charge);

struct istruct times;
inter_init(&times);

double extracted_through_contacts=0.0;
carrier_count_reset(in);
reset_np_save(in);

do
{

	if (in->time<pulse_voc_config.pulse_voc_pulse_start)
	{
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

		extracted_through_contacts=0.0;
	}else
	if ((in->time>=pulse_voc_config.pulse_voc_pulse_start)&&(in->time<(pulse_voc_config.pulse_voc_pulse_start+pulse_voc_config.pulse_voc_laser_pulse_width)))
	{
		ratio=light_convert_density(in,pulse_voc_config.pulse_voc_pulse_start,pulse_voc_config.pulse_voc_laser_pulse_width);

		if (ratio>0.0)
		{
			light_solve_and_update(in,&(in->mylight),in->Psun,ratio*pulse_voc_config.pulse_voc_laser_eff,pulse_voc_config.pulse_voc_laser_pulse_width);
		}else
		{
			light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);
		}

	}else
	{
		light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);
	}

	double i0=newton_pulse_voc(in,TRUE);

	dump_dynamic_add_data(&store,in,in->time);

	V=in->Vapplied;

	if (get_dump_status(dump_iodump)==TRUE)
	{
		printf("pulse_voc time=%e step=%d %.1e ",in->time,step,in->last_error);
		printf("Vtot=%lf current = %e mA (%e A/m^2)\n",in->Vapplied,get_I(in)/1e-3,get_J(in));
	}

	inter_append(&times,(double)ittr,in->time);

	ittr++;

	dump_write_to_disk(in);

	plot_now(in,"pulse_voc.plot");

	inter_append(&out_i,in->time,i0);
	inter_append(&out_v,in->time,get_equiv_V(in));
	inter_append(&lost_charge,in->time,extracted_through_contacts-fabs(get_extracted_n(in)+get_extracted_p(in))/2.0);

	if (pulse_voc_config.pulse_voc_use_big_mesh==TRUE)
	{
		if ((in->time>pulse_voc_config.pulse_voc_bigmeshstart)&&(in->time<pulse_voc_config.pulse_voc_bigmeshstop))
		{
			in->dt=pulse_voc_config.pulse_voc_bigdt;
		}else
		{
			in->dt=pulse_voc_config.pulse_voc_dt;
		}

	}

	if (in->time>(pulse_voc_config.pulse_voc_pulse_start+pulse_voc_config.pulse_voc_laser_pulse_width))
	{
		in->dt*=pulse_voc_config.pulse_voc_dt_mul;
	}

	device_timestep(in);
	step++;

	if (in->time>pulse_voc_config.pulse_voc_stop) break;

	stop_start(in);

}while(1);
char outpath[1000];
struct istruct out_flip;

dump_dynamic_save(in->outputpath,&store);
dump_dynamic_free(&store);

inter_copy(&out_flip,&out_i,TRUE);
inter_mul(&out_flip,-1.0);
sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i_pos.dat");
inter_save(&out_flip,outpath);

inter_free(&out_flip);


sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i.dat");
inter_save(&out_i,outpath);

inter_add_x(&out_i,pulse_voc_config.pulse_voc_shift);
sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i_norm.dat");
inter_mod(&out_i);
inter_save(&out_i,outpath);


inter_add_x(&out_v,pulse_voc_config.pulse_voc_shift);

buffer_malloc(&buf);
buf.y_mul=1.0;
buf.x_mul=1e6;
strcpy(buf.title,"Open circuit voltage - time");
strcpy(buf.type,"xy");
strcpy(buf.x_label,"Time");
strcpy(buf.y_label,"Voltage");
strcpy(buf.x_units,"$\\mu s$");
strcpy(buf.y_units,"$V$");
buf.logscale_x=0;
buf.logscale_y=0;
buffer_add_info(&buf);
buffer_add_xy_data(&buf,out_v.x, out_v.data, out_v.len);
buffer_dump(in->outputpath,"pulse_voc_v.dat",&buf);
buffer_free(&buf);


sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_lost_charge.dat");
inter_save(&lost_charge,outpath);



in->go_time=FALSE;

inter_free(&out_i);
inter_free(&out_v);
inter_free(&lost_charge);
inter_free(&times);
}

void pulse_voc_load_config(struct pulse_voc *in,struct device *dev)
{
double temp=0.0;
struct inp_file inp;
inp_init(&inp);
inp_load_from_path(&inp,dev->inputpath,"pulse_voc.inp");
inp_check(&inp,1.27);


inp_search_double(&inp,&(in->pulse_voc_dt),"#pulse_voc_dt");

inp_search_double(&inp,&(in->pulse_voc_dt_mul),"#pulse_voc_dt_mul");

inp_search_double(&inp,&(in->pulse_voc_time_start),"#pulse_voc_time_start");

inp_search_double(&inp,&(in->pulse_voc_pulse_start),"#pulse_voc_pulse_start");

inp_search_double(&inp,&(in->pulse_voc_laser_pulse_width),"#pulse_voc_laser_pulse_width");

inp_search_double(&inp,&(in->pulse_voc_stop),"#pulse_voc_stop");

inp_search_double(&inp,&(in->pulse_voc_Vexternal),"#pulse_voc_Vexternal");

inp_search_double(&inp,&(in->pulse_voc_laser_eff),"#pulse_voc_laser_eff");
in->pulse_voc_laser_eff=fabs(in->pulse_voc_laser_eff);

inp_search_double(&inp,&(in->pulse_voc_shift),"#pulse_voc_shift");

inp_search_int(&inp,&(in->pulse_voc_use_big_mesh),"#pulse_voc_use_big_mesh");

inp_search_double(&inp,&(in->pulse_voc_bigmeshstart),"#pulse_voc_bigmeshstart");

inp_search_double(&inp,&(in->pulse_voc_bigmeshstop),"#pulse_voc_bigmeshstop");

inp_search_double(&inp,&(in->pulse_voc_bigdt),"#pulse_voc_bigdt");

inp_search_double(&inp,&(temp),"#Rscope");
pulse_voc_set_Rscope(temp);

inp_free(&inp);
}
