// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../dump.h"
#include "../../sim.h"
#include "tpv.h"
#include "wire.h"

void newton_aux_tpv(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
//<clean>
double C=in->C;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
double S=1.0/(tpv_config.Rscope+in->Rcontact);
double L=tpv_config.tpv_L;
*i=i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt-(V-L*(i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt-in->ilast)/in->dt)*S;
*didv=didv0+C*(1.0)/in->dt+1.0/in->Rshunt-(1.0-L*(didv0+C*(1.0)/in->dt+1.0/in->Rshunt)/in->dt)*S;//didv0+C*(1.0)/in->dt-S;
*didphi=didphi0-(-L*(didphi0)/in->dt)*S;//didphi0;
*didxil=didxil0-(-L*(didxil0)/in->dt)*S;//didxil0;
*didxipl=didxipl0-(-L*(didxipl0)/in->dt)*S;//didxipl0;
*didphir=didphir0-(-L*(didphir0)/in->dt)*S;//didphir0;
*didxir=didxir0-(-L*(didxir0)/in->dt)*S;//didxir0;
*didxipr=didxipr0-(-L*(didxipr0)/in->dt)*S;//didxipr0;

/*
double Rdrain=1.0/(1.0/in->Rshunt+1.0/(in->Rscope+in->Rcontact));
*i=(i0+C*(V-in->Vapplied_last)/in->dt)-V/Rdrain;
*didv=(didv0+C*(1.0)/in->dt)-1.0/Rdrain;
*didphi=didphi0;
*didxil=didxil0;
*didxipl=didxipl0;
*didphir=didphir0;
*didxir=didxir0;
*didxipr=didxipr0;*/
//</clean>
return;
}

double newton_sim_tpv_fast(struct device *in,int do_LC)
{
//<clean>
in->kl_in_newton=TRUE;
solver_realloc(in);
if (do_LC==TRUE)
{
in->newton_aux=&newton_aux_tpv;
solve_all(in);
}else
{
double save_L=tpv_config.tpv_L;
double save_C=in->C;
tpv_config.tpv_L=0.0;
in->C=0.0;
in->newton_aux=&newton_aux_tpv;
solve_all(in);
tpv_config.tpv_L=save_L;
in->C=save_C;
}
return get_I(in)+in->C*(in->Vapplied-in->Vapplied_last)+in->Vapplied/in->Rshunt;
//</clean>
}

double newton_sim_tpv(struct device *in)
{
printf("Looking for Voc\n");
double C=in->C;
double clamp=0.1;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;
double Rdrain=tpv_config.Rscope+in->Rcontact;

solve_all(in);
i0=get_I(in);
e0=fabs(i0+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));

in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));

deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;

int count=0;
int max=200;
do
{
e0=e1;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));
deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
if (get_dump_status(dump_print_text)==TRUE)
{
	printf("%d TPV find Voc Vapplied=%le step=%le error=%le\n",count,in->Vapplied,step,e1);
}
if (count>max) break;
count++;

}while(e1>1e-12);

double ret=in->Vapplied-C*(i1-in->Ilast)/in->dt;
return ret;
}

double newton_sim_tpv_wire(struct device *in)
{
//<clean>
double step=0.001;
double e0;
double e1;
double i0;
double i1;
double deriv;
solve_all(in);
i0=get_I(in);
//getchar();
e0=fabs(i0+in->Vapplied/in->Rshunt+wire_sim(in->Vapplied));
in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied/in->Rshunt+wire_sim(in->Vapplied));
deriv=(e1-e0)/step;
step=-e1/deriv;
in->Vapplied+=step;
int count=0;
int max=100;
do
{
e0=e1;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied/in->Rshunt+wire_sim(in->Vapplied));
deriv=(e1-e0)/step;
step=-e1/deriv;
//if (e1<clamp) clamp=e1/10.0;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
printf("Tpv error = %e %e\n",in->Vapplied,e1);
if (count>max) break;
count++;
}while(e1>1e-8);
double cur=wire_sim(in->Vapplied);
printf("result = %e %e %e %e\n",i1,in->Vapplied/in->Rshunt,cur,in->Vapplied/cur);
//getchar();
//</clean>
return 0.0;
}

void set_light_for_voc(struct device *in,double Voc)
{
//<clean>
double clamp=100;
double step=10;
double e0;
double e1;
double deriv;

light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

newton_sim_tpv(in);
printf("Looking for Voc=%le\n",Voc);
//sim_i(in,0.0);
e0=fabs(in->Vapplied-Voc);

in->Psun+=step;
light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);
newton_sim_tpv(in);
//sim_i(in,0.0);
e1=fabs(in->Vapplied-Voc);

deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Psun+=step;

int count=0;
int max=1000;

do
{
e0=e1;
newton_sim_tpv(in);
//sim_i(in,0.0);
e1=fabs(in->Vapplied-Voc);

deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Psun+=step;
in->Psun=fabs(in->Psun);
light_solve_and_update(in,&(in->mylight),in->Psun,0.0,0.0);

printf("%d %le %le %le\n",count,e1,in->Psun,in->Vapplied);
if (count>max) break;
count++;
}while(e1>1e-5);
printf("%le %le %le #ans\n",in->Vapplied,in->Psun,get_extracted_np(in));
printf("%le %le #2ans\n",get_extracted_np(in),get_avg_recom(in)/(pow(get_extracted_np(in),2.0)));
//</clean>
}

